package com.qmth.themis.backend.quartz;

import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.dto.MqDto;
import com.qmth.themis.business.service.ProducerServer;
import com.qmth.themis.business.util.RedisUtil;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.quartz.QuartzJobBean;

import javax.annotation.Resource;
import java.util.Map;
import java.util.Objects;

/**
 * @Description: mq job
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/7/1
 */
public class MqJob extends QuartzJobBean {
    private final static Logger log = LoggerFactory.getLogger(MqJob.class);

    @Resource
    RedisUtil redisUtil;

    @Resource
    ProducerServer producerServer;

    @Override
    protected void executeInternal(JobExecutionContext context) throws JobExecutionException {
        Long size = redisUtil.getHashSize(SystemConstant.SESSION_TOPIC_ERROR_LIST);
        if (Objects.nonNull(size) && size.longValue() > 0) {
            log.info("session_topic有异常的消息数为:{}", size);
            Map map = redisUtil.getHashEntries(SystemConstant.SESSION_TOPIC_ERROR_LIST);
            map.forEach((k, v) -> {
                MqDto mqDto = (MqDto) v;
                producerServer.sendOneWay(mqDto);
            });
        }
    }
}
