package com.qmth.themis.business.constant;

import com.qmth.themis.common.contanst.Constants;
import com.qmth.themis.common.enums.Platform;
import com.qmth.themis.common.enums.Source;

import java.util.Calendar;
import java.util.Date;
import java.util.Objects;

/**
 * @Description: 系统常量
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2019/10/11
 */
public class SystemConstant {
    /**
     * 鉴权
     */
    public static final String ACCESS_TOKEN = "accessToken";
    public static final String SESSION_ID = "sessionId";
    /**
     * 阿里云oss
     */
    public static final String END_POINT = "endpoint";
    public static final String ACCESS_KEY_ID = "accessKeyId";
    public static final String ACCESS_KEY_SECRET = "accessKeySecret";
    public static final String BUCKET = "bucket";
    /**
     * 系统相关
     */
    public static final String SUCCESS = "{'success':true}";
    public static final String RECORDS = "records";
    public static final String USER = "user:";
    public static final String SESSION = "session:";
    public static final String LINK = "LINK";
    public static final String MENU = "MENU";
    public static final String AUTH_CACHE = "auth_cache";
    public static final String ALL_PATH = "/**";
    public static final String ACCOUNT = "account";
    public static final String ERROR = "/error";
    public static final String SESSION_TOPIC_ERROR_LIST = "session:topic:error:list";
    public static final String USERLOG_TOPIC_ERROR_LIST = "userLog:topic:error:list";
    /**
     * session过期时间
     */
    public static final int WEB_SESSION_EXPIRE = 1;//过期时间1天
    public static final int PC_SESSION_EXPIRE = 1;//过期时间1天
    public static final int WXAPP_SESSION_EXPIRE = 30;//过期时间30天
    /**
     * redis过期时间
     */
    public static final long REDIS_EXPIRE_TIME = 60L * 1440L;//过期时间24小时
    public static final long REDIS_REFRESH_EXPIRE_TIME = 60L * 60L;//刷新时间1小时，理论上说24小时还不睡觉也已经快不行了
    public static final long REFRESH_EXPIRE_TIME = 60L * 5L;//过期剩余时间5分钟
    /**
     * rocket mq
     */
    public static final int CONSUME_MESSAGE_BATCH_MAX_SIZE = 10;
    public static final int MAXRECONSUMETIMES = 3;
    public static final int DELIVERED_ACK_TYPE = 0;//消息"已发出"，但尚未处理结束
    public static final int POSION_ACK_TYPE = 1;//消息"错误",通常表示"抛弃"此消息，比如消息重发多次后，都无法正确处理时，消息将会被删除或者DLQ(死信队列)
    public static final int STANDARD_ACK_TYPE = 2;//"标准"类型,通常表示为消息"处理成功"，broker端可以删除消息了
    public static final int REDELIVERED_ACK_TYPE = 3;//消息需"重发"，比如consumer处理消息时抛出了异常，broker稍后会重新发送此消息
    public static final int INDIVIDUAL_ACK_TYPE = 4;//表示只确认"单条消息",无论在任何ACK_MODE下
    public static final int UNMATCHED_ACK_TYPE = 5;//BROKER间转发消息时,接收端"拒绝"消息
    public static final int UNSEND_ACK_TYPE = 6;//消息未发出

    /**
     * 获取过期时间
     *
     * @param platform
     * @return
     */
    public static Date getExpireTime(Platform platform) {
        Date now = new Date();
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(now);
        if (Objects.equals(platform.getSource(), Source.web.name())) {
            calendar.add(Calendar.DAY_OF_YEAR, SystemConstant.WEB_SESSION_EXPIRE);
        } else if (Objects.equals(platform.getSource(), Source.wxapp.name())) {
            calendar.add(Calendar.DAY_OF_YEAR, SystemConstant.WXAPP_SESSION_EXPIRE);
        } else if (Objects.equals(platform.getSource(), Source.pc.name())) {
            calendar.add(Calendar.DAY_OF_YEAR, SystemConstant.PC_SESSION_EXPIRE);
        }
        return calendar.getTime();
    }

    /**
     * 比较过期
     *
     * @param timestamp
     * @return
     */
    public static boolean expire(long timestamp) {
        long diff = (System.currentTimeMillis() - timestamp) / 1000;
        return diff < -1 * Constants.SIGNATURE_AHEAD_SECONDS || diff > Constants.SIGNATURE_EXPIRE_SECONDS;
    }
}